const defaultListFocusIdx = 1004;
const defaultDetailsFocusIdx = 2003;

function getURLParam(key) {
    if (window.location.search && window.location.search.substring(1)) {
        var params = window.location.search.substring(1).split('&');

        for (var i = 0; i < params.length; i++) {
            var pair = params[i].split('=');

            if (pair[0] == key) {
                return decodeURIComponent(pair[1]);
            }
        }
    }

    return null;
}

const app = new Vue({
    el: '#app',
    data: {
        commType: 'IP',
        view: 'list',
        focusIdx: defaultListFocusIdx,
        roomNumber: 'unknown',
        iotGroupName: '',
        hcapVersion: hcap.API_VERSION,
        frameworkStatus: 'unknown',
        thingList: [],
        zigbeeBridge: {},
        zigbeeBridgeStatus: 'unknown',
        details: {},
        iotGroups: [],
        aliases: [],
        callId: 0,
        iotGroupDialog: {
            show: false,
            focusIdx: 1,
            iotGroupName: ''
        },
        nicknameDialog: {
            show: false,
            focusIdx: 1,
            nickname: ''
        },
        dimmingDialog: {
            show: false,
            focusIdx: 1,
            data: {
                thingUid: '',
                componenetId: '',
                componenetValue: {}
            },
            dimmingValue: ''
        }
    },
    computed: {
        thingListWithKeyIdx: function () {
            var idx = 1007;

            for (var i in this.thingList) {
                this.thingList[i].keyIdx = idx;

                if (this.thingList[i].thingType === 'ThingBridge') {
                    continue;
                } else if (this.thingList[i].thingType === 'ThingDiscovered') {
                    idx += 2;
                } else {
                    idx++;
                }
            }

            return this.thingList;
        },
        thingComponentListWithKeyIdx: function () {
            const thingCompList = this.details.thingComponentList;
            var idx = 2005;

            for (var i in thingCompList) {
                if (thingCompList[i].componentSettable == true) {
                    thingCompList[i].keyIdx = idx;
                    idx++;
                }
            }

            return thingCompList;
        }
    },
    created: function () {
        this.roomNumber = getURLParam('roomnumber');
        this.commType = getURLParam('commType');

        const _this = this;
        _this.iotGroupName = localStorage.getItem('iotGroupName');
        
        if (this.commType == 'IP') {
            //axios.defaults.baseURL = 'https://' + location.hostname + ':60080';
            axios.defaults.baseURL = 'https://' + getURLParam('serverip') + ':60080';
            axios.defaults.headers.common['Authorization'] = 'Bearer ' + getURLParam('access_token');

            if (this.roomNumber) {
                axios.get('/api/v2/rooms/' + this.roomNumber)
                .then(function (res) {
                    _this.iotGroupName = res.data.data.iot.iotGroupName;

                    if (_this.iotGroupName) {
                        localStorage.setItem('iotGroupName', _this.iotGroupName);
                    }

                    _this.updateAliases();
                });

                axios.get('/api/v2/iot/groups')
                .then(function (res) {
                    if (res.data.data) {
                        _this.iotGroups = res.data.data;
                    }
                });
            }
        } else if (this.commType == 'RF') {
            var callbackUrl = getURLParam('callback');
            var iotGroupsJsonUrl = callbackUrl.substr(0, callbackUrl.lastIndexOf('/')) + '/iotGroups.json';

            axios.get(iotGroupsJsonUrl)
            .then(function (res) {
                if (res.data) {
                    _this.iotGroups = res.data;
                    _this.updateAliases();
                }
            });
        }

        MW.getFrameworkStatus({
            CB: function (resultObj) {
                if (resultObj.returnValue) {
                    _this.frameworkStatus = resultObj.status;
                    _this.getThingList();
                }
            }
        });
    },
    methods: {
        refreshPage: function () {
            window.location.href = window.location.href;
        },
        goToHome: function () {
            window.location.href = getURLParam('callback');
        },
        rebootTV: function () {
            MW.reboot({
                callId: ++this.callId,
                CB: function (resultObj) {
                    console.log(resultObj);
                }
            });
        },
        requestFactoryReset: function() {
            MW.requestFactoryReset({
                callId: ++this.callId,
                CB: function (resultObj) {
                    console.log(resultObj);
                }
            });
        },
        showList: function () {
            this.focusIdx = defaultListFocusIdx;
            this.view = 'list';
        },
        showDetails: function(thingUid) {
            this.focusIdx = defaultDetailsFocusIdx;

            for (var i in this.thingList) {
                var thing = this.thingList[i];

                if (thing.thingUid == thingUid) {
                    this.details = thing;
                    break;
                }
            }

            this.view = 'details';
        },
        updateAliases: function () {
            if (this.commType == 'IP') {
                if (this.iotGroupName) {
                    const _this = this;

                    axios.get('/api/v2/iot/groups/' + encodeURIComponent(_this.iotGroupName) + '?format=all')
                    .then(function (res) {
                        if (res.data.status == 'success') {
                            _this.aliases = res.data.data.aliases;
                            localStorage.setItem('iotScenes', JSON.stringify(res.data.data.scenes));
                        }
                    });
                }
            } else {
                this.aliases = [];

                if (this.iotGroups) {
                    for (var i = 0; i < this.iotGroups.length; i++) {
                        if (this.iotGroupName === this.iotGroups[i].name) {
                            this.aliases = this.iotGroups[i].aliases;
                            localStorage.setItem('iotScenes', JSON.stringify(this.iotGroups[i].scenes));
                            break;
                        }
                    }
                }
            }
        },
        getThingList: function() {
            const _this = this;

            if (this.frameworkStatus === 'ready') {
                MW.getThingList({
                    CB: function (resultObj) {
                        _this.thingList = resultObj.thingList;

                        for (var i in _this.thingList) {
                            if (_this.thingList[i].thingType === 'ThingBridge' && _this.thingList[i].bindingId === 'Zigbee') {
                                _this.zigbeeBridge = _this.thingList[i];
                                _this.getBridgeStatus(_this.thingList[i].thingUid);
                            }
                        }

                        _this.sendThingListToServer();
                    }
                });
            }
        },
        updateThing: function(thingUid) {
            const _this = this;

            MW.getThingList({
                thingUidList: [ thingUid ],
                CB: function (resultObj) {
                    for (var i in resultObj.thingList) {
                        var thing = resultObj.thingList[i];
                        var isUpdated = false;

                        for (var j in _this.thingList) {
                            if (thing.thingUid === _this.thingList[j].thingUid) {
                                for (var attr in thing) {
                                    if (thing.hasOwnProperty(attr)) {
                                        _this.thingList[j][attr] = thing[attr];
                                    }
                                }

                                isUpdated = true;
                                break;
                            }
                        }

                        if (!isUpdated) {
                            _this.thingList.push(thing);
                        }
                    }

                    _this.sendThingListToServer();
                }
            });
        },
        updateComponent: function(thingUid, componentId, fromValue, toValue) {
            for (var i in this.thingList) {
                if (thingUid === this.thingList[i].thingUid) {
                    for (var j in this.thingList[i].thingComponentList) {
                        if (componentId === this.thingList[i].thingComponentList[j].componentId) {
                            this.thingList[i].thingComponentList[j].componentValueType = toValue.valueType;
                            this.thingList[i].thingComponentList[j].componentValue = toValue;
                            break;
                        }
                    }

                    break;
                }
            }
        },
        removeThing: function(thingUid) {
            for (var i in this.thingList) {
                if (thingUid === this.thingList[i].thingUid) {
                    this.thingList.splice(i, 1);
                }
            }
        },
        getBridgeStatus: function(bridgeUid) {
            const _this = this;

            MW.getBridgeStatus({
                bridgeUid: bridgeUid,
                CB: function (resultObj) {
                    if (resultObj.returnValue) {
                        _this.zigbeeBridgeStatus = resultObj.status;
                    }
                }
            });
        },
        requestSetBridgeStatus: function() {
            const bridgeUid = this.zigbeeBridge.thingUid;
            const status = this.zigbeeBridgeStatus === 'scan' ? 'normal' : 'scan';

            MW.requestSetBridgeStatus({
                bridgeUid: bridgeUid,
                status: status,
                CB: function (resultObj) {
                    console.log(resultObj);
                }
            });
        },
        requestRegisterThing: function (thingUid) {
            MW.requestRegisterThing({
                thingUid: thingUid
            });
        },
        requestRejectThing: function (thingUid) {
            MW.requestRejectThing({
                thingUid: thingUid
            });
        },
        requestUnregisterThing: function (thingUid) {
            MW.requestUnregisterThing({
                thingUid: thingUid,
                callId: ++this.callId,
                CB: function (resultObj) {
                    console.log(resultObj);
                }
            });
        },
        setThingNickname: function (thingUid, nickname) {
            if (!nickname) {
                return;
            }

            MW.setThingNickname({
                thingUid: thingUid,
                nickname: nickname,
                CB: function (resultObj) {
                    console.log(resultObj);
                }
            });
        },
        requestSynchronizeThing: function (thingUid, componentId) {
            MW.requestSynchronizeThing({
                thingUid: thingUid,
                componentId: componentId
            });
        },
        requestSetComponent: function (thingUid, componentId, componentValue) {
            MW.requestSetComponent({
                thingUid: thingUid,
                componentId: componentId,
                componentValue: componentValue,
                callId: ++this.callId,
                CB: function (resultObj) {
                    console.log(resultObj);
                }
            });
        },
        openIotGroupDialog: function () {
            this.iotGroupDialog.iotGroupName = this.iotGroupName;
            this.iotGroupDialog.focusIdx = 1;
            this.iotGroupDialog.show = true;
        },
        cancelIotGroupDialog: function () {
            this.$refs.iotGroupDialogBody.scrollTop = 0;
            this.iotGroupDialog.show = false;
        },
        saveIotGroupDialog: function () {
            localStorage.setItem('iotGroupName', this.iotGroupDialog.iotGroupName);

            if (this.commType == 'IP') {
                if (this.iotGroupDialog.iotGroupName) {
                    const _this = this;

                    axios.put('/api/v2/iot/rooms', {
                        id: this.roomNumber,
                        iotGroupName: this.iotGroupDialog.iotGroupName
                    })
                    .then(function (res) {
                        if (res.data.status == 'success') {
                            _this.iotGroupName = _this.iotGroupDialog.iotGroupName;
                            _this.$refs.iotGroupDialogBody.scrollTop = 0;
                            _this.iotGroupDialog.show = false;
                        }

                        _this.updateAliases();
                    });
                } else {
                    this.$refs.iotGroupDialogBody.scrollTop = 0;
                    this.iotGroupDialog.show = false;
                }
            } else {
                this.iotGroupName = this.iotGroupDialog.iotGroupName;
                this.updateAliases();
                this.$refs.iotGroupDialogBody.scrollTop = 0;
                this.iotGroupDialog.show = false;
            }
        },
        openNicknameDialog: function() {
            this.nicknameDialog.nickname = this.details.thingMetaData.nickname;
            this.nicknameDialog.focusIdx = 1;
            this.nicknameDialog.show = true;
        },
        cancelNicknameDialog: function () {
            this.$refs.nicknameDialogBody.scrollTop = 0;
            this.nicknameDialog.show = false;
        },
        saveNicknameDialog: function () {
            if (this.details.thingMetaData.nickname != this.nicknameDialog.nickname) {
                this.setThingNickname(this.details.thingUid, this.nicknameDialog.nickname);
                this.details.thingMetaData.nickname = this.nicknameDialog.nickname;
            }

            this.$refs.nicknameDialogBody.scrollTop = 0;
            this.nicknameDialog.show = false;
        },
        openDimmingDialog: function (thingUid, componentId, componentValue) {
            this.dimmingDialog.data = {
                thingUid: thingUid,
                componentId: componentId,
                componentValue: componentValue
            };
            
            this.dimmingDialog.dimmingValue = componentValue.value;
            this.dimmingDialog.focusIdx = 1;
            this.dimmingDialog.show = true;
        },
        cancelDimmingDialog: function () {
            this.dimmingDialog.show = false;
        },
        saveDimmingDialog: function () {
            this.dimmingDialog.show = false;
        },
        sendThingListToServer: function () {
            if (this.commType == 'IP' && this.roomNumber) {
                const data = [];

                for (var i = 0; i < this.thingList.length; i++) {
                    var item = this.thingList[i];

                    if (item.thingType !== 'ThingDiscovered' && item.thingType !== 'ThingBridge') {
                        data.push({
                            id: item.thingUid,
                            name: item.thingMetaData.modelName,
                            type: item.thingType,
                            thingAliasName: item.thingMetaData.nickname,
                            value: JSON.stringify(item.thingComponentList)
                        });
                    }
                }

                if (data.length > 0) {
                    axios.post('/api/v2/rooms/' + this.roomNumber + '/things', data);
                }
            }
        }
    }
});

MW.eventCB = {
    frameworkStatusChanged: function (eventObj) {
        app.frameworkStatus = eventObj.status;
        app.getThingList();
    },
    bridgeStatusChanged: function (eventObj) {
        if (eventObj.bridgeUid == app.zigbeeBridge.thingUid) {
            app.zigbeeBridgeStatus = eventObj.status;
        }
    },
    componentReported: function (eventObj) {
        app.updateComponent(eventObj.thingUid, eventObj.componentId, eventObj.fromValue, eventObj.toValue);
    },
    thingDiscovered: function (eventObj) {
        app.updateThing(eventObj.thingUid);
    },
    thingMetaDataChanged: function (eventObj) {
        app.updateThing(eventObj.thingUid);
    },
    thingRegistered: function (eventObj) {
        app.updateThing(eventObj.thingUid);
    },
    thingRejected: function (eventObj) {
        app.removeThing(eventObj.thingUid);
    },
    thingUnregistered: function (eventObj) {
        app.showList();
        app.removeThing(eventObj.thingUid);
    },
    factoryResetResultReceived: function (eventObj) {
        console.log(eventObj);
        app.rebootTV();
    }
};

window.addEventListener("keyup", function (e) {
    if (e.keyCode == 13) { //ok
        if (app.iotGroupDialog.show) {
            document.querySelector('.btnIotGroup.selected').click();
        } else if (app.nicknameDialog.show) {
            document.querySelector('.btnNickname.selected').click();
        } else if (app.dimmingDialog.show) {
            document.querySelector('.btnDimming.selected').click();
        } else {
            document.querySelector('.main .selected').click();
        }
    } else if (e.keyCode == 37 || e.keyCode == 38) { //up
        if (app.iotGroupDialog.show) {
            if (app.iotGroupDialog.focusIdx > 1) {
                app.iotGroupDialog.focusIdx--;

                if (app.iotGroups.length > 5) {
                    var scrollTop = 0;

                    for (var i = 0; i < app.iotGroupDialog.focusIdx - 3; i++) {
                        scrollTop += app.$refs.iotGroupDialogItems[i].clientHeight;
                    }

                    app.$refs.iotGroupDialogBody.scrollTop = scrollTop;
                }
            }
        } else if (app.nicknameDialog.show) {
            if (app.nicknameDialog.focusIdx > 1) {
                app.nicknameDialog.focusIdx--;

                if (app.aliases.length > 5) {
                    var scrollTop = 0;

                    for (var i = 0; i < app.nicknameDialog.focusIdx - 3; i++) {
                        scrollTop += app.$refs.nicknameDialogItems[i].clientHeight;
                    }

                    app.$refs.nicknameDialogBody.scrollTop = scrollTop;
                }
            }
        } else if (app.dimmingDialog.show) {
            if (app.dimmingDialog.focusIdx > 1) {
                app.dimmingDialog.focusIdx--;
            }
        } else if (app.view === 'list') {
            if (app.focusIdx > 1001) {
                app.focusIdx--;
            }
        } else { // view === 'details'
            if (app.focusIdx > 2001) {
                app.focusIdx--;
            }
        }
    } else if (e.keyCode == 39 || e.keyCode == 40) { //down
        if (app.iotGroupDialog.show) {
            if (app.iotGroupDialog.focusIdx < app.iotGroups.length + 2) {
                app.iotGroupDialog.focusIdx++;

                if (app.iotGroups.length > 5) {
                    var scrollTop = 0;

                    for (var i = 0; i < app.iotGroupDialog.focusIdx - 3; i++) {
                        scrollTop += app.$refs.iotGroupDialogItems[i].clientHeight;
                    }

                    app.$refs.iotGroupDialogBody.scrollTop = scrollTop;
                }
            }
        } else if (app.nicknameDialog.show) {
            if (app.nicknameDialog.focusIdx < app.aliases.length + 2) {
                app.nicknameDialog.focusIdx++;

                if (app.aliases.length > 5) {
                    var scrollTop = 0;

                    for (var i = 0; i < app.nicknameDialog.focusIdx - 3; i++) {
                        scrollTop += app.$refs.nicknameDialogItems[i].clientHeight;
                    }

                    app.$refs.nicknameDialogBody.scrollTop = scrollTop;
                }
            }
        } else if (app.dimmingDialog.show) {
            if (app.dimmingDialog.focusIdx < 3) {
                app.dimmingDialog.focusIdx++;
            }
        } else if (app.view === 'list') {
            if (app.focusIdx < document.querySelectorAll('.main button').length + 1000) {
                app.focusIdx++;
            }
        } else { // view === 'details'
            if (app.focusIdx < document.querySelectorAll('.main button').length + 2000) {
                app.focusIdx++;
            }
        }
    }
});
