/*! dateformat.js */

var dateformat = (function(){
    var properties = {
            formatIdx : -1,
            orgFormat : "",
            showday : true,
            militaryTime : false,
            showdate : true,
            showtime : true,
            datetime : ""
        },

        /* format function strings*/
        formatFunctions = {};

    var formats = [
        { date : "M/D/YYYY", time : "hh:mm A", day : "dddd,"},
        { date : "DD/MM/YYYY", time : "HH:mm", day : "dddd"},
        { date : "DD.MM.YYYY", time : "HH:mm", day : "dddd"},
        { date : "YYYY-MM-DD", time : "hh:mm A", day : "dddd,"},
        { date : "DD.MM.YYYY", time : "HH:mm", day : "dddd,"},
        { date : "DD.M.YYYY", time : "HH:mm", day : "dddd,"},
        { date : "YYYY.MM.DD", time : "HH:mm", day : "dddd,"},
        { date : "DD/M/YYYY", time : "HH.mm", day : "dddd"},
        { date : "M/D/YYYY", time : "hh.mm A", day : "dddd,"},
        { date : "DD-M-YYYY", time : "HH:mm", day : "dddd,"},
        { date : "DD/M/YYYY", time : "HH:mm", day : "dddd,"},
        { date : "DD-M-YYYY", time : "HH:mm", day : "dddd,"},
        { date : "YYYY-MM-DD", time : "HH:mm", day : "dddd,"},
        { date : "DD.M.YYYY", time : "HH:mm", day : "dddd,"},
        { date : "DD/MM/YYYY", time : "HH:mm", day : "dddd"},
        { date : "D/M/YYYY", time : "HH:mm", day : "dddd"},
        { date : "YYYY/M/D", time : "A hh:mm", day : "dddd"},
        { date : "D/M/YYYY", time : "A hh:mm", day : "dddd"},
        { date : "YYYY/MM/DD", time : "HH:mm", day : "dddd"},
        { date : "DD:MM:YYYY", time : "HH:mm", day : "dddd"},
        { date : "YYYY:MM:DD", time : "HH:mm", day : "dddd"}
    ];
    var langFormat= {dateFormat:[{langCode:"al_AL",dateFormatId:4},{langCode:"ar_AE",dateFormatId:20},{langCode:"bs_BA",dateFormatId:14},{langCode:"bg_BG",dateFormatId:3},{langCode:"zh_CN",dateFormatId:9},{langCode:"zh_HK",dateFormatId:16},{langCode:"zh_TW",dateFormatId:16},{langCode:"bs_BA",dateFormatId:17},{langCode:"hr_HR",dateFormatId:6},{langCode:"cs_CZ",dateFormatId:14},{langCode:"da_DK",dateFormatId:15},{langCode:"nl_NL",dateFormatId:12},{langCode:"en_CA",dateFormatId:4},{langCode:"fr_CA",dateFormatId:4},{langCode:"en_GB",dateFormatId:2},{langCode:"en_US",dateFormatId:1},{langCode:"et_EE",dateFormatId:3},{langCode:"fa_IR",dateFormatId:21},{langCode:"fi_FI",dateFormatId:14},{langCode:"fr_FR",dateFormatId:2},{langCode:"de_DE",dateFormatId:5},{langCode:"el_GR",dateFormatId:2},{langCode:"he_IL",dateFormatId:20},{langCode:"hu_HU",dateFormatId:7},{langCode:"it_IT",dateFormatId:8},{langCode:"ja_JP",dateFormatId:19},{langCode:"kk_KZ",dateFormatId:3},{langCode:"ko_KR",dateFormatId:4},{langCode:"lv_LV",dateFormatId:3},{langCode:"lt_LT",dateFormatId:7},{langCode:"mk_MK",dateFormatId:3},{langCode:"no_NO",dateFormatId:3},{langCode:"pl_PL",dateFormatId:3},{langCode:"pt_BR",dateFormatId:11},{langCode:"pt_PT",dateFormatId:10},{langCode:"ro_RO",dateFormatId:3},{langCode:"ru_RU",dateFormatId:3},{langCode:"sr_RS",dateFormatId:14},{langCode:"sk_SK",dateFormatId:14},{langCode:"sl_SI",dateFormatId:3},{langCode:"es_ES",dateFormatId:2},{langCode:"sv_SE",dateFormatId:3},{langCode:"tr_TR",dateFormatId:3},{langCode:"uk_UA",dateFormatId:3}]};    var StandardFormatByLanguage = 98;
    
    function getDateFormatIdx(){
        /*return (properties.formatIdx === -1 ) ? $('a.btnLogin').data('dateformat') : properties.formatIdx;*/
        if (properties.formatIdx === -1 )
            return $('a.btnLogin').data('dateformat');
        if (properties.formatIdx === StandardFormatByLanguage){
            return getStandardDateByLanguage();
        }
        return properties.formatIdx;
    }

    function getOrgFormat(format) {
        return format.day + " " + format.date + " " + format.time;
    }

    /**
     *  default setting value
     *  for date widget property
     *  @param idx : selected date format index number
     */
    function getDefaultFormat(idx) {
        var idx = _.isUndefined(idx) ? getDateFormatIdx() : idx;
        properties.formatIdx = idx;

        var result = properties;

        result.orgFormat = getOrgFormat(formats[idx]);
        result.militaryTime = formats[idx].time.indexOf("A") !== -1 ? true : false;

        return result;
    }

    /**
     * return date format of active language
     * @param : 
     * @returns : idx of date format of active language
     */
    function getStandardDateByLanguage(){
        if (_.isUndefined(getCookie("active_language"))){
            return 1;
        }
        for (var i = 0; i < langFormat.dateFormat.length; ++i){
            if (langFormat.dateFormat[i].langCode === getCookie("active_language")) { 
                return langFormat.dateFormat[i].dateFormatId;
            }
        }
        return -1;
    }

    /**
     * return date information
     * @param : {formatIdx : -1, showday : true, militaryTime : true, showdate : true, showtime : true}
     * @returns : date string
     */
    function getDateFromSelectedFormat(prop) {
        var idx = _.get(prop, "formatIdx", "") ? prop.formatIdx : getDateFormatIdx();

        var dateFormat = formats[idx];
        var defaultProperties = properties;
        prop = _.extend(defaultProperties, prop);

        var format = getOrgFormat(dateFormat);
        var result = {};

        if(!prop.showday) {
            format = format.replace(dateFormat.day, "");
        }

        if(!prop.showdate) {
            format = format.replace(dateFormat.date, "");
        }

        if(!prop.showtime) {
            format = format.replace(dateFormat.time, "");
        } else {
            if(!_.isUndefined(prop.militaryTime)){
                var time = "";
                if(prop.militaryTime) {
                    time = dateFormat.time.replace("A", "");
                    time = $.trim(time.replace("hh", "HH"));
                } else {
                    time = dateFormat.time + (dateFormat.time.indexOf("A") != -1 ? "" : " A");
                    time = $.trim(time.replace("HH", "hh"));
                }
                format = format.replace(dateFormat.time, time);
            }
        }

        if(!prop.datetime) {
            prop.datetime = undefined;
        } else {
            var numberDateTime = _.toNumber(prop.datetime);
            if(!_.isNaN(numberDateTime)) {
                prop.datetime = numberDateTime;
            }
        }

        result.format = $.trim(format);
        if(result.format.indexOf(",", result.format.length - ",".length) >= 0){
            result.format = result.format.slice(0, -1);
        }

        result.currentTime = moment(prop.datetime).format(result.format);

        return result;
    }

    function setDateFormatIdx(idx) {
        if($.isNumeric(idx) && (idx <= formats.length || idx === StandardFormatByLanguage)) { properties.formatIdx = idx; }
        //properties.formatIdx = idx;
    }

    return {
        langFormat : langFormat,
        setDateFormatIdx : setDateFormatIdx,
        getDateFormat : getDefaultFormat,
        getDate: getDateFromSelectedFormat
    };
})();
